/***********************************************************************************************************
Copyright (c) 2012, Sebastian Breß, Otto-von-Guericke University of Magdeburg, Germany. All rights reserved.

This program and accompanying materials are made available under the terms of the 
GNU LESSER GENERAL PUBLIC LICENSE - Version 3, http://www.gnu.org/licenses/lgpl-3.0.txt
***********************************************************************************************************/
#pragma once

#include <iostream>
#include <string>
#include <vector>
#include <memory>

//loki library
//#include <loki/Singleton.h>
#include <loki/Factory.h>

//STEMOD includes
#include <core/time_measurement.hpp>
#include <config/global_definitions.hpp>

namespace hype{
	namespace core{

class Algorithm; //forward declaration

class RecomputationHeuristic{
	public:
	bool recompute(Algorithm& algorithm);
	virtual bool internal_recompute(Algorithm& algorithm) = 0;
	virtual ~RecomputationHeuristic(){}

protected:
	RecomputationHeuristic(const std::string& name);

private:
	std::string name_;
	unsigned int length_of_training_phase_;
	bool is_initial_approximation_function_computed_;
	unsigned int samplecounter_;
};

//factory function
const std::tr1::shared_ptr<RecomputationHeuristic> getNewRecomputationHeuristicbyName(std::string name);

typedef Loki::Factory< RecomputationHeuristic, std::string> RecomputationHeuristicFactory; //aFactory;
//typedef Loki::Singleton<RecomputationHeuristicFactory> RecomputationHeuristicFactorySingleton;


class RecomputationHeuristicFactorySingleton{
	public:
		static RecomputationHeuristicFactory& Instance();
};

	}; //end namespace core
}; //end namespace stemod


