/***********************************************************************************************************
Copyright (c) 2013, Sebastian Breß, Otto-von-Guericke University of Magdeburg, Germany. All rights reserved.

This program and accompanying materials are made available under the terms of the 
GNU LESSER GENERAL PUBLIC LICENSE - Version 3, http://www.gnu.org/licenses/lgpl-3.0.txt
***********************************************************************************************************/
#pragma once

#include <stdlib.h>
#include <stdio.h>

#include <config/global_definitions.hpp>

//#define NDEBUG //uncomment for release version
#include <assert.h>

 #ifdef __cplusplus
 extern "C" {
 #endif


//enum C_ComputeDevice {CPU,GPU,FPGA,NP}; //CPU,GPU,FPGA, Network Processor

typedef struct C_SchedulingDecision{
	void* cpp_scheduling_decison_object_ptr; //=NULL;
} C_SchedulingDecision;

typedef struct C_AlgorithmMeasurement{
	unsigned long long starting_point_timestamp; //=0;
	int is_valid; //=0;
	C_SchedulingDecision* scheduling_decision_ptr; //=NULL;
} C_AlgorithmMeasurement;

int hype_add_algorithm_to_operation(const char* name_of_operation, const char* name_of_algorithm, 
												enum ComputeDevice comp_dev, const char* name_of_statistical_method, 
											   const char* name_of_recomputation_strategy);
												
int hype_set_optimization_criterion(const char* name_of_operation,const char* name_of_optimization_criterion);

int hype_set_statistical_method(const char* name_of_algorithm,const char* name_of_statistical_method);

int hype_set_recomputation_heuristic(const char* name_of_algorithm,const char* name_of_recomputation_strategy);

char* hype_scheduling_decision_get_algorithm_name(C_SchedulingDecision* scheduling_decision_ptr);

enum ComputeDevice hype_scheduling_decision_get_compute_device(C_SchedulingDecision* scheduling_decision_ptr);

double hype_scheduling_decision_get_estimated_executiontime(C_SchedulingDecision* scheduling_decision_ptr);

C_SchedulingDecision* hype_get_scheduling_decision(const char* name_of_operation, const double* feature_vector, size_t number_of_features);


void hype_before_algorithm_execution(struct C_AlgorithmMeasurement* algmeas,C_SchedulingDecision* scheduling_decision_ptr, const double* values,size_t number_of_values); //starts timer

void hype_after_algorithm_execution(struct C_AlgorithmMeasurement* algmeas); //stops timer

int hype_add_observation(C_SchedulingDecision* scheduling_decision_ptr, double measured_execution_time_in_ns);


 #ifdef __cplusplus
 }
 #endif
