

#pragma once

//#include <core/scheduling_decision.hpp>

//#include <boost/shared_ptr.hpp>

#include <query_processing/binary_operator.hpp>

namespace hype
{
	namespace queryprocessing
	{

		template <typename OperatorInputTypeLeftChild, typename OperatorInputTypeRightChild,typename OperatorOutputType>
		class BinaryOperator : public TypedOperator<OperatorOutputType>{ //uses 2D Learning Method
			public:
			//typedef typename OperatorMapper_Helper_Template<OperatorOutputType>::TypedOperator TypedOperator;
			typedef typename OperatorMapper_Helper_Template<OperatorOutputType>::TypedOperatorPtr TypedOperatorPtr;
			typedef typename OperatorMapper_Helper_Template<OperatorOutputType>::TypedNodePtr TypedNodePtr;			
			
			BinaryOperator(const hype::SchedulingDecision& sched_dec, 
								boost::shared_ptr<TypedOperator<OperatorInputTypeLeftChild> > left_child, 
								boost::shared_ptr<TypedOperator<OperatorInputTypeRightChild> >  right_child)
				: TypedOperator<OperatorOutputType>(sched_dec), left_child_(left_child), right_child_(right_child) {}


			virtual ~BinaryOperator(){}

			OperatorInputTypeLeftChild getInputDataLeftChild(){
				return this->left_child_->getResult();
			}

			OperatorInputTypeRightChild getInputDataRightChild(){
				return this->right_child_->getResult();
			}

			virtual bool run(){
				if(left_child_){
					left_child_->run();
				}
				if(right_child_){
					right_child_->run();	
				}
				//execute this operator
				return (*this)();
			}

			virtual void print(unsigned int tree_level) const{
				for(unsigned int i=0;i<tree_level;++i){
					std::cout << "\t";
				}
				assert(!this->getFeatureValues().empty());
				std::cout << this->getAlgorithmName() << " Cost: " << this->getEstimatedExecutionTime().getTimeinNanoseconds() 
							 << "\tEstimated Cardinality of Input Data: " << this->getFeatureValues()[0]
							 << std::endl; // << " Features: " << this->getFeatureValues() << std::endl;
				if(left_child_) left_child_->print(tree_level+1);
				if(right_child_) right_child_->print(tree_level+1);
			}

			/*
			virtual std::list<TypedOperator<OperatorOutputType>& > getOperatorQueue(){
				
				std::list<TypedOperator<OperatorOutputType>& > left_list = left_child_->getOperatorQueue();
				left_list.push_front(*this);

				std::list<TypedOperator<OperatorOutputType>& > right_list = right_child_->getOperatorQueue();
						
				left_list.insert(left_list.end(),right_list.begin(),right_list.end());
				return left_list;
				//this->left_child_->
			}*/

			private:
			boost::shared_ptr<TypedOperator<OperatorInputTypeLeftChild> > left_child_;
			boost::shared_ptr<TypedOperator<OperatorInputTypeRightChild> > right_child_;
			//boost::shared_ptr<TypedOperator<OperatorOutputType> > succsessor_;
		};
		
		
	}; //end namespace queryprocessing
}; //end namespace stemod


