
#include <config/configuration.hpp>

#include <core/algorithm.hpp>
#include <core/recomputation_heuristic.hpp>


namespace hype{
	namespace core{

		RecomputationHeuristic::RecomputationHeuristic(const std::string& name) : name_(name), length_of_training_phase_(Runtime_Configuration::instance().getTrainingLength()),is_initial_approximation_function_computed_(false),samplecounter_(1) {}

		bool RecomputationHeuristic::recompute(Algorithm& algorithm){
			if(is_initial_approximation_function_computed_){
				return this->internal_recompute(algorithm);
			}else{
				if(!quiet && verbose && debug)
				{
					std::cout << "[DEBUG]: initial Training phase of algortihm '" << algorithm.getName()
								 << "' step " << samplecounter_ << "/" << length_of_training_phase_ << std::endl;
				}
				if(samplecounter_++>=length_of_training_phase_){
					is_initial_approximation_function_computed_=true;
					return true;
				}else{
					return false;
				}				
			}
		}

		const std::tr1::shared_ptr<RecomputationHeuristic> getNewRecomputationHeuristicbyName(std::string name_of_recomputation_heuristic){
			//RecomputationHeuristicFactorySingleton::Instance().Register()
			//RecomputationHeuristicFactorySingleton::Instance()
			RecomputationHeuristic* ptr = RecomputationHeuristicFactorySingleton::Instance().CreateObject(name_of_recomputation_heuristic); //.( 1, createProductNull );
			return std::tr1::shared_ptr<RecomputationHeuristic> (ptr);
		}

		RecomputationHeuristicFactory& RecomputationHeuristicFactorySingleton::Instance()
		{
			static RecomputationHeuristicFactory factory;
			return factory;
		}
	}; //end namespace core
}; //end namespace stemod

