


#include <list>

#include <config/global_definitions.hpp>
#include <core/scheduling_decision.hpp>
#include <query_processing/virtual_processing_device.hpp>
#include <util/get_name.hpp>

namespace hype
{
	namespace queryprocessing
	{
	using namespace std;
	using namespace core;
	using namespace util;	
		
VirtualProcessingDevice::VirtualProcessingDevice(const core::DeviceSpecification& dev_spec) : dev_spec_(dev_spec), scheduled_tasks_()
{
}

bool VirtualProcessingDevice::addRunningOperation(const SchedulingDecision& sched_dec)
{
	//cout << "VirtualProcessingDevice: add operator" << endl;
	scheduled_tasks_.push_back(sched_dec);
	return true;
}

double VirtualProcessingDevice::getEstimatedFinishingTime() const
{		
		TaskQueue::const_iterator it;
		double result=0;
		for(it=scheduled_tasks_.begin();it!=scheduled_tasks_.end();++it){
				result+=it->getEstimatedExecutionTimeforAlgorithm().getTimeinNanoseconds();
		}
		return result;
}

unsigned int VirtualProcessingDevice::getNumberOfRunningOperations() const
{
	return scheduled_tasks_.size();
}

bool VirtualProcessingDevice::isIdle() const
{
	return scheduled_tasks_.empty();
}

bool VirtualProcessingDevice::removeFinishedOperation(const SchedulingDecision& sched_dec)
{
	//cout << "VirtualProcessingDevice: delete operator" << endl;
	scheduled_tasks_.remove(sched_dec);
	return true;
	
}

const core::DeviceSpecification& VirtualProcessingDevice::getDeviceSpecification() const throw(){
		return this->dev_spec_;
}

void VirtualProcessingDevice::print() const throw(){
		cout << "Processing Device " <<  dev_spec_.getProcessingDeviceID() << ":" << endl;
		cout << "\t" << "Device Type: " << getName(dev_spec_.getDeviceType()) << endl;
		cout << "\t" << "MemoryID: " << dev_spec_.getMemoryID() << endl;		
		cout << "\t" << "Number of Running Operations: "<< this->getNumberOfRunningOperations() << endl;
		cout << "\t" << "Estimated Finishing Time: " << this->getEstimatedFinishingTime() << endl;
		cout << "\t" << "Is Idle: " << this->isIdle() << endl;
}

	}; //end namespace queryprocessing
}; //end namespace hype
